use Test::More;

use Linux::Capabilities;

is CAP_CHOWN, 0;
is CAP_DAC_OVERRIDE, 1;
is CAP_DAC_READ_SEARCH, 2;
is CAP_FOWNER, 3;
is CAP_FSETID, 4;
is CAP_KILL, 5;
is CAP_SETGID, 6;
is CAP_SETUID, 7;
is CAP_SETPCAP, 8;
is CAP_LINUX_IMMUTABLE, 9;
is CAP_NET_BIND_SERVICE, 10;
is CAP_NET_BROADCAST, 11;
is CAP_NET_ADMIN, 12;
is CAP_NET_RAW, 13;
is CAP_IPC_LOCK, 14;
is CAP_IPC_OWNER, 15;
is CAP_SYS_MODULE, 16;
is CAP_SYS_RAWIO, 17;
is CAP_SYS_CHROOT, 18;
is CAP_SYS_PTRACE, 19;
is CAP_SYS_PACCT, 20;
is CAP_SYS_ADMIN, 21;
is CAP_SYS_BOOT, 22;
is CAP_SYS_NICE, 23;
is CAP_SYS_RESOURCE, 24;
is CAP_SYS_TIME, 25;
is CAP_SYS_TTY_CONFIG, 26;
is CAP_MKNOD, 27;
is CAP_LEASE, 28;
is CAP_AUDIT_WRITE, 29;
is CAP_AUDIT_CONTROL, 30;
is CAP_SETFCAP, 31;
is CAP_MAC_OVERRIDE, 32;
is CAP_MAC_ADMIN, 33;
is CAP_SYSLOG, 34;
is CAP_WAKE_ALARM, 35;
is CAP_BLOCK_SUSPEND, 36;
is CAP_AUDIT_READ, 37;

is CAP_EFFECTIVE, 0;
is CAP_PERMITTED, 1;
is CAP_INHERITABLE, 2;

is CAP_CLEAR, 0;
is CAP_SET, 1;

done_testing;